//-------------------------------------------------------------------------
// Titel : LCD_DISPLAY MIT 16BIT TIMER
//-------------------------------------------------------------------------
// Funktion :  LCD_DISPLAY MIT 16BIT TIMER

//			   
//-------------------------------------------------------------------------
// Prozessor : ATmega168
// Takt : 14745600 Hz
// Sprache : C
// Datum : 01.09.2010

//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
// Include
//-------------------------------------------------------------------------



#include <inttypes.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <string.h>

#include <stdio.h>
#include <stdlib.h>


#include "uart.h"

#include "irmp.h"
#include "irmpconfig.h"

#include "irsndconfig.h"
#include "irsnd.h"


//-------------------------------------------------------------------------
// Definitionen
//-------------------------------------------------------------------------



/* define CPU frequency in Mhz here if not defined in Makefile */
#ifndef F_CPU
#define F_CPU 14745600UL
#endif

/* 115200 baud */
#define UART_BAUD_RATE      115200      


//-------------------------------------------------------------------------
// INTERRUPTVEKTOR 
//-------------------------------------------------------------------------

//----------------------------------------------
// ISR(TIMER1_COMPA_vect)
//----------------------------------------------

ISR(TIMER1_COMPA_vect)
{
 	if (! irsnd_ISR())          // call irsnd ISR
  	{                           // if not busy...
    	PORTC &= ~(1<<PC1);
		irmp_ISR();             // call irmp ISR
  	}
   	else
	{
		PORTC |= (1<<PC1);
	}

}



//-------------------------------------------------------------------------
// Funktionesprototypen
//-------------------------------------------------------------------------
void  ALLini ();
void  PORTini ();
void timer_init (void);
void uart_gets( char* Buffer, uint8_t MaxLen );

//-------------------------------------------------------------------------
// Funktionen
//-------------------------------------------------------------------------



//----------------------------------------------
// ALGEMEIN INIT
//----------------------------------------------

void  ALLini ()
{

	PORTini();

    irmp_init();                                                              // initialize rc5
  	irsnd_init();                                                             // initialize irsnd 
	timer_init();
	uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU) );
	
	sei();

	_delay_ms(10000);
	 uart_puts("IR_BORD_ONLINE\n");
}

//----------------------------------------------
// PORTini
//----------------------------------------------
void  PORTini ()
{
	DDRC |= (1<<PC1); // PC1  als Ausgang

}

//----------------------------------------------
// timer_init
//----------------------------------------------


void timer_init (void)
{
 	OCR1A   =  (F_CPU / F_INTERRUPTS) - 1;                                    // compare value: 1/10000 of CPU frequency
	TCCR1B  = (1 << WGM12) | (1 << CS10);                                     // switch CTC Mode on, set prescaler to 1
	#if defined (__AVR_ATmega8__) || defined (__AVR_ATmega16__) || defined (__AVR_ATmega32__) || defined (__AVR_ATmega64__) || defined (__AVR_ATmega162__) 
  		TIMSK  = 1 << OCIE1A;                                                     // OCIE1A: Interrupt by timer compare
	#else
  		TIMSK1  = 1 << OCIE1A;                                                    // OCIE1A: Interrupt by timer compare
	#endif  // __AVR...
}


//----------------------------------------------
// UART DATEN IN EINEN STRING UMWANDELN
//---------------------------------------------- 
void uart_gets( char* Buffer, uint8_t MaxLen )

{
	uint8_t StringLen = 0; 							// Untervarialben StringLen auf 0 Setzen
	unsigned int c;									// Untervarialben C
	c = uart_getc();								// Hole ein Zeichen und ein INFO von  uart_getc (UART.H)
	if ( c & UART_NO_DATA )							// IST DER RINGBUFFER LEHR
	{
		*Buffer = '\0';								// Dann Sring enden einfgen

	}
    else											// Sonst
    {
       	while( c!='\0' && StringLen < MaxLen - 1 )  // Solange Char Sammen bis (new line) oder string lnge eintrift 
		{
			*Buffer++ =c;  							// Buffer Zeiger +1 das neue zeichen einfgen
			_delay_ms(0.5);    						// kurz warten
			c = uart_getc(); 						// Hole ein Zeichen
    		//uart_putc (c);
			StringLen++; 							// StringLen +1
		}
		*Buffer = '\0';  							// Und am schuss ein 00 fr Stirng ende

		
			
	}

}


//-------------------------------------------------------------------------
//Main 
//-------------------------------------------------------------------------

int main()

{
	char ircodeout[30];
	char ircodein[30];
	char trennzeichen[] = "," ;// Weiter Zerlegen mit den trennzeichen ,

	IRMP_DATA irmp_data;
    ALLini();

   
	while(1)
	{
	irsnd_send_data (&irmp_data, TRUE);
	if (irmp_get_data (&irmp_data))
    	{
       	 	sprintf (ircodeout,"%d,%d,%d\n",irmp_data.protocol,irmp_data.address,irmp_data.command);
			uart_puts(ircodeout);
				irmp_data.protocol=0;
				irmp_data.address=0;
				irmp_data.command=0;
	    }

		else
		{

			uart_gets( ircodein, sizeof( ircodein ) ); 			
			irmp_data.protocol = atoi (strtok(ircodein, trennzeichen)); 
			if (irmp_data.protocol !=0) 
			{
				irmp_data.address = atoi (strtok(NULL, trennzeichen));
				irmp_data.command = atoi (strtok(NULL, trennzeichen)); 
				irmp_data.flags = atoi (strtok(NULL, trennzeichen));
 				irsnd_send_data (&irmp_data, TRUE);
				sprintf (ircodein,"%d,%d,%d\n",irmp_data.protocol,irmp_data.address,irmp_data.command);
				uart_puts(ircodein);
				irmp_data.protocol=0;
				irmp_data.address=0;
				irmp_data.command=0;
					
			}
		}

	}

}


